@@ -11,102 +11,84 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
+ * License along with GEGL; if not, see <https://www.gnu.org/licenses/>.
  *
  * Copyright 2006 Øyvind Kolås <pippin@gimp.org>
+ * Copyright 2013 Daniel Sabo
  */
 
 #include "config.h"
 #include <glib/gi18n-lib.h>
 
-#ifdef GEGL_CHANT_PROPERTIES
+#ifdef GEGL_PROPERTIES
 
-gegl_chant_double (value, _("Opacity"), -10.0, 10.0, 1.0,
-         _("Global opacity value that is always used on top of the optional auxiliary input buffer."))
+property_double (value, _("Opacity"), 1.0)
+    description (_("Global opacity value that is always used on top of the optional auxiliary input buffer."))
+    value_range (-10.0, 10.0)
+    ui_range    (0.0, 1.0)
 
 #else
 
-#define GEGL_CHANT_TYPE_POINT_COMPOSER
-#define GEGL_CHANT_C_FILE       "opacity.c"
+#define GEGL_OP_POINT_COMPOSER
+#define GEGL_OP_NAME     opacity
+#define GEGL_OP_C_SOURCE opacity.c
 
-#include "gegl-chant.h"
+#include "gegl-op.h"
 
 #include <stdio.h>
+#include <math.h>
 
+#define EPSILON 1e-6f
 
 static void
 prepare (GeglOperation *self)
 {
+  const Babl *space = gegl_operation_get_source_space (self, "input");
   const Babl *fmt = gegl_operation_get_source_format (self, "input");
-  GeglChantO *o = GEGL_CHANT_PROPERTIES (self);
 
-  if (fmt)
-    {
-      const Babl *model = babl_format_get_model (fmt);
+  fmt = gegl_babl_variant (fmt, GEGL_BABL_VARIANT_ALPHA);
 
-      if (model == babl_model ("RaGaBaA") ||
-          model == babl_model ("R'aG'aB'aA") ||
-          model == babl_model ("YaA") ||
-          model == babl_model ("Y'aA"))
-        {
-          o->chant_data = NULL;
-
-          gegl_operation_set_format (self, "input",
-                                     babl_format ("RaGaBaA float"));
-          gegl_operation_set_format (self, "output",
-                                     babl_format ("RaGaBaA float"));
-          gegl_operation_set_format (self, "aux",
-                                     babl_format ("Y float"));
-
-          return;
-        }
-    }
-
-  /* ugly way of communicating that we want the RGBA version */
-  /* because of that, we can't use the common opencl api for point ops */
-  o->chant_data = (void*)0xabc;
-
-  gegl_operation_set_format (self, "input", babl_format ("RGBA float"));
-  gegl_operation_set_format (self, "output", babl_format ("RGBA float"));
-  gegl_operation_set_format (self, "aux", babl_format ("Y float"));
+  gegl_operation_set_format (self, "input", fmt);
+  gegl_operation_set_format (self, "output", fmt);
+  gegl_operation_set_format (self, "aux", babl_format_with_space ("Y float", space));
 
   return;
 }
 
 static void
-process_RaGaBaAfloat (GeglOperation       *op,
+process_premultiplied_float (GeglOperation       *op,
                       void                *in_buf,
                       void                *aux_buf,
                       void                *out_buf,
                       glong                samples,
                       const GeglRectangle *roi,
-                      gint                 level)
+                      gint                 level,
+                      gint                 components)
 {
   gfloat *in = in_buf;
   gfloat *out = out_buf;
   gfloat *aux = aux_buf;
-  gfloat value = GEGL_CHANT_PROPERTIES (op)->value;
+  gfloat value = GEGL_PROPERTIES (op)->value;
 
   if (aux == NULL)
     {
-      g_assert (value != 1.0); /* buffer should have been passed through */
       while (samples--)
         {
           gint j;
-          for (j=0; j<4; j++)
+          for (j=0; j<components; j++)
             out[j] = in[j] * value;
-          in  += 4;
-          out += 4;
+          in  += components;
+          out += components;
         }
     }
-  else if (value == 1.0)
+  else if (fabsf (value - 1.0f) <= EPSILON)
     while (samples--)
       {
         gint j;
-        for (j=0; j<4; j++)
+        for (j=0; j<components; j++)
           out[j] = in[j] * (*aux);
-        in  += 4;
-        out += 4;
+        in  += components;
+        out += components;
         aux += 1;
       }
   else
@@ -114,50 +96,51 @@ process_RaGaBaAfloat (GeglOperation       *op,
       {
         gfloat v = (*aux) * value;
         gint j;
-        for (j=0; j<4; j++)
+        for (j=0; j<components; j++)
           out[j] = in[j] * v;
-        in  += 4;
-        out += 4;
+        in  += components;
+        out += components;
         aux += 1;
       }
 }
 
 static void
-process_RGBAfloat (GeglOperation       *op,
-                   void                *in_buf,
-                   void                *aux_buf,
-                   void                *out_buf,
-                   glong                samples,
-                   const GeglRectangle *roi,
-                   gint                 level)
+process_with_alpha_float (GeglOperation       *op,
+                          void                *in_buf,
+                          void                *aux_buf,
+                          void                *out_buf,
+                          glong                samples,
+                          const GeglRectangle *roi,
+                          gint                 level,
+                          gint                 components)
 {
   gfloat *in = in_buf;
   gfloat *out = out_buf;
   gfloat *aux = aux_buf;
-  gfloat value = GEGL_CHANT_PROPERTIES (op)->value;
+  gint ccomponents = components - 1;
+  gfloat value = GEGL_PROPERTIES (op)->value;
 
   if (aux == NULL)
     {
-      g_assert (value != 1.0); /* buffer should have been passed through */
       while (samples--)
         {
           gint j;
-          for (j=0; j<3; j++)
+          for (j=0; j<ccomponents; j++)
             out[j] = in[j];
-          out[3] = in[3] * value;
-          in  += 4;
-          out += 4;
+          out[ccomponents] = in[ccomponents] * value;
+          in  += components;
+          out += components;
         }
     }
-  else if (value == 1.0)
+  else if (fabsf (value - 1.0f) <= EPSILON)
     while (samples--)
       {
         gint j;
-        for (j=0; j<3; j++)
+        for (j=0; j<ccomponents; j++)
           out[j] = in[j];
-        out[3] = in[3] * (*aux);
-        in  += 4;
-        out += 4;
+        out[ccomponents] = in[ccomponents] * (*aux);
+        in  += components;
+        out += components;
         aux += 1;
       }
   else
@@ -165,11 +148,11 @@ process_RGBAfloat (GeglOperation       *op,
       {
         gfloat v = (*aux) * value;
         gint j;
-        for (j=0; j<3; j++)
+        for (j=0; j<ccomponents; j++)
           out[j] = in[j];
-        out[3] = in[3] * v;
-        in  += 4;
-        out += 4;
+        out[ccomponents] = in[ccomponents] * v;
+        in  += components;
+        out += components;
         aux += 1;
       }
 }
@@ -183,10 +166,13 @@ process (GeglOperation       *op,
          const GeglRectangle *roi,
          gint                 level)
 {
-  if (GEGL_CHANT_PROPERTIES (op)->chant_data != NULL)
-    process_RGBAfloat (op, in_buf, aux_buf, out_buf, samples, roi, level);
+  const Babl *format = gegl_operation_get_format (op, "output");
+  int components = babl_format_get_n_components (format);
+
+  if (babl_get_model_flags (format) & BABL_MODEL_FLAG_ASSOCIATED)
+    process_premultiplied_float (op, in_buf, aux_buf, out_buf, samples, roi, level, components);
   else
-    process_RaGaBaAfloat (op, in_buf, aux_buf, out_buf, samples, roi, level);
+    process_with_alpha_float (op, in_buf, aux_buf, out_buf, samples, roi, level, components);
 
   return TRUE;
 }
@@ -209,6 +195,7 @@ cl_process (GeglOperation       *op,
   cl_int cl_err = 0;
   int kernel;
   gfloat value;
+  const Babl *fmt;
 
   if (!cl_data)
     {
@@ -217,9 +204,10 @@ cl_process (GeglOperation       *op,
     }
   if (!cl_data) return TRUE;
 
-  value = GEGL_CHANT_PROPERTIES (op)->value;
+  fmt = gegl_operation_get_format(op, "input");
+  value = GEGL_PROPERTIES (op)->value;
 
-  kernel = (GEGL_CHANT_PROPERTIES (op)->chant_data == NULL)? 0 : 1;
+  kernel = ((babl_get_model_flags(fmt) & BABL_MODEL_FLAG_ASSOCIATED) == 0)? 1 : 0;
 
   cl_err = gegl_clSetKernelArg(cl_data->kernel[kernel], 0, sizeof(cl_mem),   (void*)&in_tex);
   CL_CHECK;
@@ -252,13 +240,14 @@ static gboolean operation_process (GeglOperation        *operation,
 {
   GeglOperationClass  *operation_class;
   gpointer in, aux;
-  operation_class = GEGL_OPERATION_CLASS (gegl_chant_parent_class);
+  gfloat value = GEGL_PROPERTIES (operation)->value;
+  operation_class = GEGL_OPERATION_CLASS (gegl_op_parent_class);
 
   /* get the raw values this does not increase the reference count */
   in = gegl_operation_context_get_object (context, "input");
   aux = gegl_operation_context_get_object (context, "aux");
 
-  if (in && !aux && GEGL_CHANT_PROPERTIES (operation)->value == 1.0)
+  if (in && !aux && fabsf (value - 1.0f) <= EPSILON)
     {
       gegl_operation_context_take_object (context, "output",
                                           g_object_ref (G_OBJECT (in)));
@@ -273,7 +262,7 @@ static gboolean operation_process (GeglOperation        *operation,
 
 
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass              *operation_class;
   GeglOperationPointComposerClass *point_composer_class;
@@ -291,6 +280,8 @@ gegl_chant_class_init (GeglChantClass *klass)
   gegl_operation_class_set_keys (operation_class,
     "name"       , "gegl:opacity",
     "categories" , "transparency",
+    "title",       _("Opacity"),
+    "reference-hash", "b20e8c1d7bb20af95f724191feb10103",
     "description",
           _("Weights the opacity of the input both the value of the aux"
             " input and the global value property."),
